% ------------------ a percent sign makes the rest of line a comment -------------
%
%
% (0)  This file should work well with MatLab and Octave 3.6+
%
% (1)  At the prompt, you may change directory by typing
%      cd directory_name (e.g. cd e:\mystuff) and hit Return
%
% (2)  Don't know in which directory you are? Type pwd and hit Return.
%
% (3)  To see the files in the current directory, type dir and hit Return.
%
% (4)  Change this file in a text editor. Matlab has a built-in editor, so you can
%      just type edit MathLec.m and hit Return. Many versions of Octave do not have
%      an editor, so you need an external program (eg. Notepad, which is on every windows machine).
%
% (5)  To execute this program from the command line: type MathLec and hit Return.
%
% (6)  To clear the memory between runs of the program: type clear and hit Return.
%
% (7)  Leave MatLab or Octave: type exit and hit Return.
%
%
%  Paul Sderlind (Paul.Soderlind at unisg.ch), changed May 2014
%-------------------------------------------------------------------------

clear                         %clear memory
clc                           %clear screen
close all                     %close all plot windows (if any)

diary('MathLec.out')                %printing: to screen AND the text file MathLec.out


disp(' ')
disp('------------------- create and print matrix, some math -------------------')
disp(' ')                         %disp(x) prints x, where x is a matrix or
                                  %a string (text) is like 'dogs are better than cats'

Q = 1;                            %create a scalar, semi-colon supresses printing of result
q = [ 1,2,3;                      %create 2x3 matrix
      4,5,6 ];
disp('Q is a scalar. To print, use disp() or skip the semi-colon');disp(Q);      %print matrix
Q                                 %another way of printing (no semi-colon: prints).

disp(' ');disp('q is a matrix' ...            %Command ends at line break, unless line ends with ...
);disp(q);                        %case sensitive (q and Q are different)

disp(' ');disp('first line of q');disp(q(1,:))
disp(' ');disp('second column of q');disp(q(:,2))

p   = 1:10:21;                    %creating sequence of 1,11,21 as row vector
disp(' ');disp('p is a row vector'); disp(p);

z   = p';                         %transposing
disp(' ');disp('z is the transpose of p'); disp(z);

z   = [q; p];                      %overwriting old z with new, stacking q and p vertically
disp(' ');disp('stacking q and p vertically'); disp(z);

z2   = [q, q/10];                %stacking q(1,:) and p horizontally
disp(' ');disp('stacking q and q/10 horizontally'); disp(z2);


w  = [ 10,11,12;                  %create 2x3 matrix
       13,14,15 ];
disp(' ');disp('q (again, so you do not forget it)');disp(q);
disp(' ');disp('w, another matrix');disp(w);

y1 = q + w;                       %matrix addition
disp(' ');disp('q + w');disp(y1);

y2 = q.*w;                             %element-by-element multiplication
disp(' ');disp('q.*w');disp(y2);



disp(' ')                    %print a blank line
disp('----------------------  Load data from ascii file ---------------')
disp(' ')

%The following is a portion of MathLecData.txt (percentage signs added here):
%1861   10.094   9.998   -999.99
%1862   10.128   9.994   0.037
%1863   10.156  10.044   0.092
%1864   10.176  10.061   0.098


x   = load('MathLecData.txt');    %loading into matrix, semicolon (;) to suppress printing
vvi = (x == -999.99);             %logical 1 if -999.99
x(vvi) = NaN;                     %set all -999.99 to NaN
disp('First five lines of x:');disp(x(1:5,:))    %several commands on one line separated by semicolon

x2   = dlmread('MathLecData.txt');   %a perhaps better way to load data from file
clear x2                             %clearing x2 from memory, as we do not need it

year = x(:,1);                    %picking out first column
lnY  = ...                        %commands continue until line break, but
       x(:,2);                    %add ... at the end of line to make
lnC  = x(:,3);                    %the command continue on the next line
R    = x(:,4);                    %lnY and lnC are just names


disp(' ')
disp('-------------------  OLS and random numbers -------------------')
disp(' ')                         %disp(x) prints x and a lin

ett = ones( size(lnY,1),1 );      %a vector with ones, no. rows from variable
x = [ett, lnY];                   %x is Tx2 matrix
y   = lnC;                        %just to get standard OLS notation
b   = inv(x'*x)*x'*y;             %OLS. x'x doesn't work; use x'*x
b2  = x\y;                        %also OLS, much quicker and numerically more stable
u   = y - x*b;                    %OLS residuals
R2  = 1 - var(u)/var(y);
disp(' ');disp('OLS coefficients, regressing lnC on constant and lnY, two different calculations')
disp([b,b2])
disp(' ');disp('R2');
disp(R2)
disp(' ');disp('no. of observations'); disp(size(lnY,1))


x = randn(100,3);                 %matrix of random draws from N(0,1)
disp(' ');disp('Mean and Std of random draws from N(0,1): ')
disp(mean(x))           %mean of each column in matrix, gives row vector
disp(std(x))


disp(' ')
disp('---------------------- Writing a loop ---------------------------------')
disp(' ')

i = 1;
x = 0;
while i <= 10
  x = x + i;
  disp('iteration and x value');disp([i,x]);
  i = i + 1;
end


disp(' ')
disp('-------------------- Writing nested loops -----------------------------')
disp(' ')
disp('Calculate vech(x) of square matrix x')

x = reshape(1:9,3,3);             %generate square matrix
[m,n] = size(x);                  %find no. rows and columns

v = NaN(n*(n+1)/2,1);             %to put results in
k = 1;
for j = 1:n                       %loop over columns in x
  for i = j:n                     %loop over rows on and below diagonal
    v(k) = x(i,j);
    k = k + 1;                    %update index in v
  end
end

disp(' ');disp('Original matrix: ')
disp(x)
disp(' ');disp('vech of the matrix (stack elements on and below diagonal): ');
disp(v)


disp(' ')
disp('---------------------- Functions --------------------------------------')
disp(' ')

x = 1;
y = MathLecB(x,2,3);                  %calling on function in file MathLecB.m
disp('result from the function MathLecB (defined in a separate file)');
disp(y)                               %a function must be defined in its own file
                                      %one function per file, same name as function


disp(' ')
disp('------------------------------- Graphs ---------------------------------------')
disp('Notice: graphs are displayed in separate windows')
disp(' ')

t = (-3:6/99:6)';

figure
  hb = plot( t,MathLecC(t),t,3*cos(t) );
  set(hb,{'LineStyle'},{'--';'-'})
  set(hb,{'Color'},{'b';'r'})
  set(hb,{'Linewidth'},{5;1})
  legend('From MathLecC','3cos(t)')
  xlabel('t')
  ylabel('my output value')
  text(-2.5,0.9,'some text')
  title('Result from MathLecC and 3cos(t)')
  print('-dpng',   'MathLec1.png')                  %print to file
  %print('-djpeg', 'MathLec1.jpg');                 %try also these
  %print('-depsc2','MathLec1.eps');
  %print('-dpdf',  'MathLec1.pdf');
                                      %notice: Octave sometimes print better after you execute
                                      %"graphics_toolkit('gnuplot')"


figure
  hb = plot( t,MathLecD(t,1),t,MathLecD(t,0.5) );
  set(hb,{'LineStyle'},{'--';'-'})
  set(hb,{'Color'},{'b';'r'})
  set(hb,{'Linewidth'},{5;1})
  legend('MathLecD(t,1)','MathLecD(t,0.5)')
  xlabel('t')
  print('-dpng','MathLec2.png')


disp(' ')
disp('----------------- solving (non-linear) equations ----------------------')
disp(' ')

x1 = fzero(@MathLecC,[-1,1]);            %fzero finds a root of a function, try between -1 and 1
disp('at which x is MathLecC(x) = 0? The function fzero finds it.')
disp(x1)
x1 = fzero(@MathLecC,[1,3]);             %now, try between 1 and 3
disp('at which x is MathLecC(x) = 0? Yes, there are several roots. Just look at it (in graph).')
disp(x1)


disp(' ')
disp('-------------------------- optimization -------------------------------')
disp(' ')

x1 = fminsearch(@MathLecC,1);                  %minimum, starting guess of 1
disp('argmin MathLecC, fminsearch finds it')   %in Octave you may need to install the 'optim' package
disp(x1)



disp(' ')
disp('----- optimization when function has extra arguments, more advanced ----')
disp(' ')

Loss_par1 = @(par)MathLecD(par,1);        %an anonymous function, second argument is 1
x1 = fminsearch(Loss_par1,1);
Loss_par2 = @(par)MathLecD(par,0.5);      %an anonymous function, second argument is 0.5
x2 = fminsearch(Loss_par2,1);
disp('argmin MathLecD(x,1) and argmin MathLecD(x,0.5)')
disp([x1,x2])


disp(' ')
disp('------------------------- load xls file -------------------------------')
disp(' ')

if exist('OCTAVE_VERSION','builtin')      %if you have Octave
  disp('Octave has no built-in xls reader. Save xls file as to text tile and then import into Octave.')
else                                      %if you have MatLab
  x2  = xlsread('MathLecData.xls');       %loading xls file into matrix
  vvi = (x2 == -999.99);                  %find indices of all -999.99
  x2(vvi) = NaN;                          %all -999.99 to NaN
  disp('First five lines of x2:');disp(x2(1:5,:));
end


disp(' ')
disp('------------------ More advanced graphs -------------------------------')
disp(' ')

b = linspace(-3,3,20)';
c = linspace(1,7,25)';

loss1 = 2*b.^2;
loss2 = NaN(length(b),length(c));        %mesh(c,b,loss2) wants loss2 to be length(b) x length(c)
for j = 1:length(c)                      %create loss2 column by column
  loss2(:,j) = 2*b.^2 + (c(j)-4)^2 - 0.0*b.*(c(j)-4);
end

figure
subplot(2,2,1)                %do subplot 1 on 2x2 grid
  plot(b,loss1)
  axis([-3,3,0,20])
  title('Line plot')
subplot(2,2,2)
  bar(b,loss1)
  axis([-3,3,0,20])
  title('Bar plot')
subplot(2,2,3)
  stairs(b,loss1)
  axis([-3,3,0,20])
  title('Stairs plot')
subplot(2,2,4)
  mesh(c,b,loss2)           %note the order of c and b
  axis([1,7,-3,3,0,30])
  title('Mesh plot')
print('-dpng','MathLec3.png')



disp(' ')
disp('---------------------- Handle NaNs ----------------------')
disp(' ')

z = [lnC,R];
if any(any(isnan(z)))       %check if any NaNs
  disp(' ');disp('lnC or R has some missing values. Cannot do calculations on those.')
end
disp('For instance, if data has NaNs, then most calculations give a NaN.');
disp('Standard deviations of Tx2 matrix z ');disp(std(z))

disp('');disp('Cut out row 1, since the NaN is there')
disp('');disp('Standard deviations of (T-1)x2 matrix z(2:end,:) ');disp(std(z(2:end,:)))

                                  %a more automatic approach
vv = any(isnan(z),2);             %1 if row has any NaN
z(vv,:) = [];                     %replace those rows with 'empty' (that is, cut out those rows)
disp(' ');disp('Standard deviations of automatically trimmed matrix ');disp(std(z))


disp(' ')
disp('---------------------- cleaning up after program ----------------------')
disp(' ')

diary off                         %close the output file

